@@ -5,6 +5,8 @@ import requests
 import re
 import logging
 
+from . import utils
+
 gAllVMs = None
 
 def get_all_vms(params):
@@ -38,7 +40,7 @@ def print_details(token):
     print("")
     print("Please re-run twigs with appropriate values for subscription, resource group and workspace.")
     print("")
-    sys.exit(1)
+    utils.tw_exit(0)
 
 # Main entry point
 def get_inventory(args):
@@ -59,7 +61,7 @@ def get_inventory(args):
     if args.azure_subscription is None or args.azure_resource_group is None or args.azure_workspace is None:
         print_details(token)
         return
-    assets = retrieve_inventory(params)
+    assets = retrieve_inventory(args, params)
     return assets
 
 def parse_inventory(email,data,params):
@@ -187,7 +189,7 @@ def get_os_type(ostype):
     logging.debug("Mapped OS [%s] to Asset Type [%s]", ostype, asset_type)
     return asset_type
 
-def retrieve_inventory(params):
+def retrieve_inventory(args, params):
     email = params['handle']
     sub_id = params['subscription']
     resource_group = params['resource_group']
@@ -199,7 +201,10 @@ def retrieve_inventory(params):
 
     logging.info("Retrieving inventory details from Azure...") 
 
-    resp = requests.post(url, headers=headers, json=json_data)
+    if args.insecure:
+        resp = requests.post(url, headers=headers, json=json_data, verify=False)
+    else:
+        resp = requests.post(url, headers=headers, json=json_data)
     if resp.status_code == 200:
         response = resp.json()
         if response.get('tables'):
@@ -208,7 +213,7 @@ def retrieve_inventory(params):
     else:
         logging.error("Error could not get asset inventory details from Azure...")
         logging.error("Response content: %s" % resp.text)
-        sys.exit(1)
+        utils.tw_exit(1)
 
 #Get access token using  an AAD, an app id associted with that AAD and the API key/secret for that app
 def get_access_token(params, resource):
@@ -226,7 +231,7 @@ def get_access_token(params, resource):
     else:
         logging.error("Error unable to get access token for API calls")
         logging.error("Response content: %s" % resp.text)
-        sys.exit(1)
+        utils.tw_exit(1)
 
     return token
 
@@ -275,7 +280,7 @@ def get_os_details(host, vmuuid, params):
         else:
             return False, None, None
     else:
-        logging.warn("Warning unable to get OS version details for VM [%s]. It might not be running..." % host)
+        logging.warning("Warning unable to get OS version details for VM [%s]. It might not be running..." % host)
         return False, None, None
 
 # Get details for all VMs
@@ -311,7 +316,7 @@ def get_all_subscriptions(token):
     else:
         logging.error("API call to get all subscriptions failed")
         logging.error("Response content: %s" % resp.text)
-        sys.exit(1)
+        utils.tw_exit(1)
     return allsubs
 
 #Get all resource groups for a subscription
@@ -328,7 +333,7 @@ def get_all_resourcegroups_for_subscription(subid,token):
     else:
         logging.error("API call to get all resource groups failed")
         logging.error("Response content: %s" % resp.text)
-        sys.exit(1)
+        utils.tw_exit(1)
 
     return allresourcegroups
 
@@ -347,7 +352,7 @@ def get_all_workspaces_for_subscription(subid,token):
     else:
         logging.error("API call to get all workspaces failed")
         logging.error("Response content: %s" % resp.text)
-        sys.exit(1)
+        utils.tw_exit(1)
 
     return allworkspaces
 
