@@ -27,8 +27,11 @@ def make_temp_directory(tmp_dir):
     return temp_dir
 
 def on_rm_error( func, path, exc_info):
-    os.chmod( path, stat.S_IWRITE )
-    os.unlink( path )
+    try:
+        os.chmod( path, stat.S_IWRITE )
+        os.unlink( path )
+    except Exception:
+        return
 
 # Note this routine only handles direct symbolic links and not nested ones
 def fix_symbolic_links_helper(container_fs):
@@ -41,15 +44,23 @@ def fix_symbolic_links_helper(container_fs):
                 if os.path.isabs(symlink_path) and not symlink_path.startswith(container_fs):
                     nested_path = container_fs + symlink_path
                     if os.path.exists(nested_path):
-                        # Fix symbolic link to point within container_fs
-                        os.remove(file_path)
-                        os.symlink(nested_path, file_path)
+                        try:
+                            # Fix symbolic link to point within container_fs
+                            os.remove(file_path)
+                            os.symlink(nested_path, file_path)
+                        except Exception as e:
+                            logging.warning("Unable to fix symbolic link ["+file_path+"]")
+                            logging.warning(str(e))
 
 def fix_symbolic_links(container_fs):
-    # This is a two phase operation as there could be symbolic links that could not be processed
-    # due to a symbolic that will be processed later...
-    fix_symbolic_links_helper(container_fs)
-    fix_symbolic_links_helper(container_fs)
+    try:
+        # This is a two phase operation as there could be symbolic links that could not be processed
+        # due to a symbolic that will be processed later...
+        fix_symbolic_links_helper(container_fs)
+        fix_symbolic_links_helper(container_fs)
+    except Exception as e:
+        logging.warning("Encountering issue while fixing symbolic links")
+        logging.warning(str(e))
 
 def tar_available():
     if os.path.isfile("/bin/tar"):
@@ -399,10 +410,14 @@ def discover_container_from_image(args, digest):
         logging.info("Retrieving container filesystem")
         if args.image is not None:
             container_tar = save_image(args, temp_dir)
+            utils.update_tool_run_record()
             container_fs = get_container_fs(container_tar)
+            utils.update_tool_run_record()
         else:
             container_tar = export_container(args, temp_dir)
+            utils.update_tool_run_record()
             container_fs = unpack_container_fs(container_tar)
+            utils.update_tool_run_record()
 
         if container_fs is None:
             logging.warning("Unable to analyze container filesystem")
@@ -433,6 +448,7 @@ def discover_container_from_image(args, digest):
             plist = discover_ubuntu_from_container_image(container_fs)
         elif atype == 'Alpine Linux':
             plist = discover_alpine_from_container_image(container_fs)
+        utils.update_tool_run_record()
 
         shutil.rmtree(temp_dir, onerror = on_rm_error)
         if plist is None or len(plist) == 0:
@@ -756,9 +772,9 @@ def get_inventory(args, digest=None):
 
     global docker_cli
 
-    if os.geteuid() != 0:
-        logging.error("Docker operations need root privilege. Please run as 'sudo' or 'root'")
-        return None
+#    if os.geteuid() != 0:
+#        logging.error("Docker operations need root privilege. Please run as 'sudo' or 'root'")
+#        return None
 
     docker_cli = docker_available()
     if not docker_cli:
@@ -788,7 +804,7 @@ def get_inventory(args, digest=None):
     # If image digest is available, then check if image has changed using digest
     if digest is not None and digest != -1:
         if (args.token is None or len(args.token) == 0):
-            logging.warn("Unable to compare image digest as [token] argument is not specified")
+            logging.warning("Unable to compare image digest as [token] argument is not specified")
         else:
             no_change = False
             digest_tag = "IMAGE_DIGEST:%s" % digest
