@@ -16,6 +16,8 @@ import zipfile
 from xml.dom import minidom
 import toml
 import re
+import ast
+import textwrap
 
 from . import utils as lib_utils
 from . import code_secrets as lib_code_secrets
@@ -29,7 +31,7 @@ if GIT_PATH is None:
     else:
         GIT_PATH = '/usr/bin/git'
 
-SUPPORTED_TYPES = ['pip', 'ruby', 'yarn', 'nuget', 'npm', 'maven', 'gradle', 'dll', 'jar', 'cargo', 'go']
+SUPPORTED_TYPES = ['pip', 'ruby', 'yarn', 'nuget', 'npm', 'maven', 'gradle', 'dll', 'jar', 'cargo', 'go', 'composer']
 
 def cleanse_semver_version(pv):
     pv = pv.replace('"','')
@@ -50,6 +52,31 @@ def cleanse_semver_version(pv):
             pv = pv + " " + temp_tokens[token_index]
     return pv
 
+def discover_composer(args, localpath):
+    plist = []
+    files = lib_utils.find_files(localpath, 'composer.lock')
+    if len(files) <= 0:
+        files = lib_utils.find_files(localpath, 'composer.json')
+    for file_path in files:
+        fp = lib_utils.tw_open(file_path, args.encoding)
+        if fp == None:
+            continue
+        contents = fp.read()
+        fp.close()
+        composer_json = json.loads(contents)
+        if file_path.endswith('composer.json'):
+            requires = composer_json['require'] if 'require' in composer_json else {}
+            for pname in requires.keys():
+                pver = requires[pname]
+                prod = pname + " " + pver + " source:"+file_path
+                plist.append(prod)
+        else:
+            packages = composer_json['packages']
+            for p in packages:
+                prod = p['name'] + ' ' + p['version'] + ' source:'+file_path
+                plist.append(prod)
+    return plist, None
+
 def discover_go_mod(args, localpath):
     plist = []
     files = lib_utils.find_files(localpath, 'go.mod')
@@ -369,8 +396,8 @@ def discover_package_json(args, localpath):
             logging.debug("Filtering out unused npm dependencies. This may take some time...")
             plist = filter_used_npm_dependencies(args, plist, localpath)
         else:
-            logging.warn("Including unused dependencies")
-            logging.warn("May increase false positives")
+            logging.warning("Including unused dependencies")
+            logging.warning("May increase false positives")
     return plist, p1list
 
 def discover_packages_config(args, localpath):
@@ -451,8 +478,8 @@ def discover_packages_config(args, localpath):
             logging.debug("Filtering out unused dotnet dependencies. This may take some time...")
             plist = filter_used_dotnet_dependencies(args, plist, localpath)
         else:
-            logging.warn("Including unused dependencies")
-            logging.warn("May increase false positives")
+            logging.warning("Including unused dependencies")
+            logging.warning("May increase false positives")
 
     if len(plist) > 0:
         return plist, plist
@@ -485,8 +512,8 @@ def discover_packages_config(args, localpath):
             logging.debug("Filtering out unused nuget dependencies. This may take some time...")
             plist = filter_used_dotnet_dependencies(args, plist, localpath)
         else:
-            logging.warn("Including unused dependencies")
-            logging.warn("May increase false positives")
+            logging.warning("Including unused dependencies")
+            logging.warning("May increase false positives")
 
     return plist, plist
 
@@ -610,6 +637,89 @@ def discover_python(args, localpath):
         except:
             logging.error("Unable to parse python dependencies")
             continue
+    """
+    files = lib_utils.find_files(localpath, 'setup.py')
+    for file_path in files:
+        #Parse setup.py and return args and keywords args to its setup
+        #function call
+
+        mock_setup = textwrap.dedent('''\
+        def setup(*args, **kwargs):
+            __setup_calls__.append((args, kwargs))
+        ''')
+        parsed_mock_setup = ast.parse(mock_setup, filename=file_path)
+        with open(file_path, 'rt') as setup_file:
+            parsed = ast.parse(setup_file.read())
+            for index, node in enumerate(parsed.body[:]):
+                if (
+                    not isinstance(node, ast.Expr) or
+                    not isinstance(node.value, ast.Call)
+                    #not isinstance(node.value, ast.Call) or
+                    #node.value.func.id != 'setup'
+                ):
+                    continue
+                parsed.body[index:index] = parsed_mock_setup.body
+                break
+
+        fixed = ast.fix_missing_locations(parsed)
+        codeobj = compile(fixed, file_path, 'exec')
+        local_vars = {}
+        global_vars = {'__setup_calls__': []}
+        cwd = os.getcwd()
+        os.chdir(os.path.dirname(file_path))
+        try:
+            exec(codeobj, global_vars, local_vars)
+        except Exception:
+            # move on
+            logging.warning("Unable to parse python dependencies %s", file_path)
+            os.chdir(cwd)
+            continue
+        os.chdir(cwd)
+        reqs = global_vars['__setup_calls__'][0][1]['install_requires']
+        for r in reqs:
+            prod = r
+            prod = prod.replace("==", " ")
+            prod = prod.replace(">=", " ")
+            prod = prod.replace("<=", " ")
+            prod = prod.replace("~=", " ")
+            prod = prod.replace(">", " ")
+            prod = prod.replace("<", " ")
+            prod = prod + " source:" + file_path
+            if prod not in plist:
+                plist.append(prod)
+    """
+
+    files = lib_utils.find_files(localpath, 'pyproject.toml')
+    for file_path in files:
+        fp = lib_utils.tw_open(file_path, args.encoding)
+        if fp == None:
+            continue
+        contents = fp.read()
+        tdict = toml.loads(contents)
+        fp.close()
+        if localpath.startswith('/tmp/'):
+            file_path = file_path.replace(localpath+'/','')
+        if 'tool' in tdict and 'poetry' in tdict['tool'] and 'dependencies' in tdict['tool']['poetry']:
+            for d in tdict['tool']['poetry']['dependencies']:
+                ver = tdict['tool']['poetry']['dependencies'][d]
+                if isinstance(ver, dict):
+                    ver = ver['version']
+                ver = cleanse_semver_version(ver)
+                prod = d + " " + ver + " source:" + file_path
+                if prod not in plist:
+                    plist.append(prod)
+        if 'project' in tdict and 'dependencies' in tdict['project']: 
+            for d in tdict['project']['dependencies']:
+                prod = d 
+                prod = prod.replace("==", " ")
+                prod = prod.replace(">=", " ")
+                prod = prod.replace("<=", " ")
+                prod = prod.replace("~=", " ")
+                prod = prod.replace(">", " ")
+                prod = prod.replace("<", " ")
+                prod = prod + " source:" + file_path
+                if prod not in plist:
+                    plist.append(prod)
     return plist, None
 
 def LOWORD(dword):
@@ -666,7 +776,7 @@ def discover_jar(args, localpath):
                 #print "Error: No manifest found"
                 pass
         except zipfile.BadZipfile:
-            logging.warn("Unable to inspect file: %s", os.path.basename(file_path))
+            logging.warning("Unable to inspect file: %s", os.path.basename(file_path))
 
         if prod == '' or ver == '':
             jfile = os.path.basename(file_path)
@@ -719,6 +829,8 @@ def discover_specified_type(repo_type, args, localpath):
         plist, p1list = discover_cargo_toml(args, localpath)
     elif repo_type == 'go':
         plist, p1list = discover_go_mod(args, localpath)
+    elif repo_type == 'composer':
+        plist, p1list = discover_composer(args, localpath)
 
     return plist, p1list
 
@@ -801,7 +913,8 @@ def discover_inventory(args, localpath):
     asset_data['tags'] = asset_tags
     if len(tech2prod_dict) > 0:
         asset_data['compliance_metadata'] = {"source_metadata": {"technology_products":tech2prod_dict, "shallow_technology_products":shallow_tech2prod_dict}}
-    
+   
+    lib_utils.update_tool_run_record()
     return [ asset_data ]
 
 # Note this error routine assumes that the file was read-only and hence could not be deleted
@@ -821,13 +934,17 @@ def get_inventory_helper(args):
         base_path = path
         new_repo = None
         logging.info("Cloning repo locally...")
+        out = ""
         try:
             if args.branch and args.branch != '':
                 cmdarr = [GIT_PATH, 'clone', '--branch', args.branch, args.repo, path+'/.']
             else:
                 cmdarr = [GIT_PATH, 'clone', args.repo, path+'/.']
-            out = subprocess.check_output(cmdarr, stderr=dev_null_device)
-        except:
+            #out = subprocess.check_output(cmdarr, stderr=dev_null_device)
+            out = subprocess.check_output(cmdarr)
+        except Exception as e:
+            logging.error(out)
+            logging.error(str(e))
             logging.error(traceback.format_exc())
             logging.error('Error cloning repo locally')
             shutil.rmtree(path, onerror = on_rm_error)
@@ -846,17 +963,20 @@ def get_inventory_helper(args):
         logging.info("Discovering secrets/sensitive information. This may take some time.")
         secret_records = lib_code_secrets.scan_for_secrets(args, path, base_path)
         assets[0]['secrets'] = secret_records
+    lib_utils.update_tool_run_record()
 
     code_issues = []
     if args.sast:
         logging.info("Performing static analysis. This may take some time.")
         sast_records = sast.run_sast(args, path, base_path)
         code_issues.extend(sast_records)
+    lib_utils.update_tool_run_record()
 
     if args.iac_checks:
         logging.info("Identifying infrastructure as code (IaC) issues. This may take some time.")
         iac_records = iac.run_iac_checks(args, path, base_path)
         code_issues.extend(iac_records)
+    lib_utils.update_tool_run_record()
 
     if len(code_issues) > 0:
         assets[0]['sast'] = code_issues
