@@ -86,11 +86,19 @@ except (ImportError,ValueError):
     from twigs import policy as policy_lib
     from twigs.__init__ import __version__
 
-def export_assets_to_file(assets, json_file):
-    logging.info("Exporting assets to JSON file [%s]", json_file)
+def export_assets_to_sbom_file(assets, timestamp, args):
+    json_file = args.sbom
+    logging.info("Exporting assets to SBOM JSON file [%s]", json_file)
+    sbom_json = { }
+    sbom_json['meta'] = { }
+    sbom_json['meta']['generated_by'] = args.handle
+    sbom_json['meta']['generated_on'] = str(timestamp)
+    sbom_json['meta']['tool_name'] = 'twigs'
+    sbom_json['meta']['tool_version'] = __version__
+    sbom_json['assets'] = assets
     with open(json_file, "w") as fd:
-        json.dump(assets, fd, indent=2, sort_keys=True)
-    logging.info("Successfully exported assets to JSON file!")
+        json.dump(sbom_json, fd, indent=2, sort_keys=True)
+    logging.info("Successfully exported assets to SBOM JSON file!")
 
 def push_asset_to_TW(asset, args):
     asset_url = "https://" + args.instance + "/api/v2/assets/"
@@ -153,10 +161,10 @@ def run_scan(asset_id_list, pj_json, args):
         run_lic_scan = True
         if pj_json is not None:
             for policy in pj_json['policy_json']:
-                if policy['type'] == 'vulnerability':
+                if policy['type'] == 'cicdvulnpolicy':
                     logging.info("Impact assessment performed as part of policy evaluation")
                     run_va_scan = False # VA scan already done, so don't do it again
-                elif policy['type'] == 'license':
+                elif policy['type'] == 'cicdlicensepolicy':
                     logging.info("License compliance performed as part of policy evaluation")
                     run_lic_scan = False # License scan already done, so don't do it again
 
@@ -164,7 +172,6 @@ def run_scan(asset_id_list, pj_json, args):
         if run_va_scan:
             # Start VA
             scan_payload = { }
-            scan_payload['scan_type'] = 'full' 
             scan_payload['assets'] = asset_id_list
             # if args.purge_assets:
             #    scan_payload['mode'] = 'email-purge'
@@ -220,6 +227,10 @@ def add_asset_location(assets, location):
     for asset in assets:
         asset['location'] = location
 
+def add_asset_timestamp(assets, timestamp):
+    for asset in assets:
+        asset['timestamp'] = str(timestamp)
+
 def add_asset_criticality_tag(assets, asset_criticality):
     asset_criticality_tag = 'CRITICALITY:'+str(asset_criticality)
     add_asset_tags(assets, [asset_criticality_tag])
@@ -231,7 +242,7 @@ def sub_pkg_get_inventory(args):
     except pkg_resources.DistributionNotFound:
         logging.error("Error required package [%s] is not installed", dist)
         logging.error("Please install using command [sudo pip install %s]", dist)
-        sys.exit(1)
+        utils.tw_exit(1)
     module = importlib.import_module('%s.%s' % (dist,dist))
     get_inventory_func = getattr(module, "get_inventory")
     assets = get_inventory_func(args)
@@ -324,6 +335,9 @@ def main(args=None):
         parser.add_argument('--handle', help='The ThreatWorx registered email of the user. Note this can set as "TW_HANDLE" environment variable', required=False)
         parser.add_argument('--token', help='The ThreatWorx API token of the user. Note this can be set as "TW_TOKEN" environment variable', required=False)
         parser.add_argument('--instance', help='The ThreatWorx instance. Note this can be set as "TW_INSTANCE" environment variable')
+        parser.add_argument('--run_id', help='Specify a distinct identifier for this twigs discovery run')
+        # Hidden argument to track the record identifier for a run
+        parser.add_argument('--run_record_id', help=argparse.SUPPRESS)
         parser.add_argument('--location', help='Specify location information for discovered asset(s).')
         parser.add_argument('--create_empty_asset', action='store_true', help='Create empty asset even if nothing is discovered. Applicable to source code (repo) assets.')
         parser.add_argument('--tag_critical', action='store_true', help='Tag the discovered asset(s) as critical')
@@ -332,7 +346,8 @@ def main(args=None):
         parser.add_argument('--no_auto_tags', action='store_true', help='Disable auto tagging of assets with standard classification tags. Only user specified tags will be applied')
         #parser.add_argument('--asset_criticality', choices=['1', '2', '3','4', '5'], help='Business criticality of the discovered assets on a scale of 1 (low) to 5 (high).', required=False)
         parser.add_argument('--apply_policy', help='One or more policy names as a comma-separated list', required=False)
-        parser.add_argument('--out', help='Specify name of the JSON file to hold the exported asset information.')
+        parser.add_argument('--sbom', help='Specify name of the SBOM file to hold the exported asset information.')
+        parser.add_argument('--out', help=argparse.SUPPRESS)
         parser.add_argument('--no_scan', action='store_true', help='Do not initiate a baseline assessment')
         parser.add_argument('--email_report', action='store_true', help='After impact refresh is complete email scan report to self')
         group = parser.add_mutually_exclusive_group()
@@ -539,7 +554,8 @@ def main(args=None):
         parser_repo.add_argument('--branch', help='Optional branch of remote git repo')
         parser_repo.add_argument('--type', choices=repo.SUPPORTED_TYPES, help='Type of open source component to scan for. Defaults to all supported types if not specified', required=False)
         parser_repo.add_argument('--level', help='Possible values {shallow, deep}. Shallow restricts discovery to 1st level dependencies only. Deep discovers dependencies at all levels. Defaults to shallow discovery if not specified', choices=['shallow','deep'], required=False, default='shallow')
-        parser_repo.add_argument('--include_unused_dependencies', action='store_true', help='Include unused dependencies in the repository asset (applies to certain types of open source components only, may introduce false positives if used)')
+        parser_repo.add_argument('--include_unused_dependencies', action='store_true', default=True, help=argparse.SUPPRESS)
+        #parser_repo.add_argument('--include_unused_dependencies', action='store_true', help='Include unused dependencies in the repository asset (applies to certain types of open source components only, may introduce false positives if used)')
         parser_repo.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
         parser_repo.add_argument('--assetname', help='A name/label to be assigned to the discovered asset')
         # Switches related to secrets scan for repo
@@ -635,8 +651,9 @@ def main(args=None):
 
         # Arguments required for Host discovery on Linux
         parser_linux = subparsers.add_parser ("host", help = "Discover linux host assets")
-        parser_linux.add_argument('--remote_hosts_csv', help='CSV file containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
-        parser_linux.add_argument('--host_list', help='Same as the option: remote_hosts_csv. A file (currently in CSV format) containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
+        #parser_linux.add_argument('--remote_hosts_csv', help='CSV file containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.', help=argparse.SUPPRESS)
+        parser_linux.add_argument('--remote_hosts_csv', help=argparse.SUPPRESS)
+        parser_linux.add_argument('--host_list', help='A file (currently in CSV format) containing details of remote hosts. CSV file column header [1st row] should be: hostname,userlogin,userpwd,privatekey,assetid,assetname. Note "hostname" column can contain hostname, IP address, CIDR range.')
         parser_linux.add_argument('--secure', action='store_true', help='Use this option to encrypt clear text passwords in the host list file')
         parser_linux.add_argument('--password', help='A password used to encrypt / decrypt login information from the host list file')
         parser_linux.add_argument('--assetid', help='A unique ID to be assigned to the discovered asset')
@@ -653,8 +670,8 @@ def main(args=None):
         parser_vmware.add_argument('--password', help='Password for the vCenter user. Note this can be set as "VCENTER_PASSWD" environment variable')
 
         # Arguments required for nmap discovery
-        parser_nmap = subparsers.add_parser ("nmap", help = "Discover assets using nmap")
-        parser_nmap.add_argument('--hosts', help='A hostname, IP address or CIDR range', required=True)
+        parser_nmap = subparsers.add_parser ("nmap", help = "Discover endpoints and services as assets using nmap")
+        parser_nmap.add_argument('--hosts', help='A hostname, IP address or CIDR range')
         parser_nmap.add_argument('--no_ssh_audit', action='store_true', help='Skip ssh audit')
 
         # Arguments required for SBOM-based discovery
@@ -752,6 +769,8 @@ def main(args=None):
         parser_o365.add_argument('--application_key', help='O365 Application Key', required=True)
 
         args = parser.parse_args()
+        if args.out is not None:
+            args.sbom  = args.out
 
         logging_level = logging.WARNING
         if args.verbosity >= 1:
@@ -827,17 +846,22 @@ def main(args=None):
 #        logging.error('Purge assets option (--purge_assets) is used with Email report (--email_report)')
 #        sys.exit(1)
 
-        if (args.token is None or len(args.token) == 0) and args.out is None:
+        if (args.token is None or len(args.token) == 0) and args.sbom is None:
             logging.error('[token] argument is not specified and [out] argument is not specified. Unable to share discovered assets.')
             sys.exit(1)
 
-        if args.schedule is not None and sys.platform != 'win32':
+        if sys.platform != 'win32' and args.schedule is not None:
             from crontab import CronSlices
             # validate schedule
             if CronSlices.is_valid(args.schedule) == False:
                 logging.error("Error: Invalid cron schedule [%s] specified!" % args.schedule)
                 sys.exit(1)
 
+        utils.set_run_args(args)
+        response = utils.create_new_tool_run_record()
+        if response is not None and response.status_code != 200:
+            utils.tw_exit(1)
+
         assets = []
         sub_pkg_list = ['ssl_audit']
         if args.mode in sub_pkg_list:
@@ -902,6 +926,7 @@ def main(args=None):
         if args.mode != 'host' or args.secure == False:
             if assets is None or len(assets) == 0:
                 logging.info("No assets found!")
+                utils.update_tool_run_record('WARNING')
             else:
                 """
                 if args.asset_criticality is not None:
@@ -923,8 +948,13 @@ def main(args=None):
                 if args.location:
                     add_asset_location(assets, args.location)
 
-                if args.out is not None:
-                    export_assets_to_file(assets, args.out)
+                timestamp = int(time.time())
+                if args.mode != "sbom":
+                    # SBOM assets will have their own timestamp
+                    add_asset_timestamp(assets, timestamp)
+
+                if args.sbom is not None:
+                    export_assets_to_sbom_file(assets, timestamp, args)
 
                 if args.token is not None and len(args.token) > 0:
                     asset_id_list, scan_asset_id_list = push_assets_to_TW(assets, args)
@@ -973,17 +1003,20 @@ def main(args=None):
                         njob = user_cron.new(command=cron_cmd, comment=cron_comment)
                         njob.setall(args.schedule)
                         logging.info("Added to crontab with comment [%s]", cron_comment)
+                utils.update_tool_run_record('SUCCESS')
+        else:
+            utils.update_tool_run_record('SUCCESS')
 
         logging.info('Run completed')
         if exit_code is not None:
             logging.info("Exiting with code [%s] based on policy evaluation", exit_code)
-            sys.exit(exit_code)
+            sys.exit(int(exit_code))
 
     except Exception as e:
         logging.error("Something went wrong with this twigs run")
         st = traceback.format_exc()
         logging.error("Exception trace details: %s", st)
-        sys.exit(1)
+        utils.tw_exit(1)
 
 if __name__ == '__main__':
     main()
