@@ -8,7 +8,13 @@ import logging
 import requests
 import time
 
+try:
+    from .__init__ import __version__
+except (ImportError,ValueError):
+    from twigs.__init__ import __version__
+
 GoDaddyCABundle = True
+run_args = None
 
 SYSTEM_TAGS = ['IMAGE_NAME', 'OS_VERSION', 'OS_RELEASE', 'SOURCE', 'OS_RELEASE_ID', 'CRITICALITY', 'http', 'https', 'OS_ARCH', 'IMAGE_DIGEST', 'DISCOVERY_TYPE']
 
@@ -226,6 +232,8 @@ def get_asset_type(os):
         return "Windows"
     elif "alpine" in os:
         return "Alpine Linux"
+    elif "container-optimized os" in os:
+        return "Google Container-Optimized OS"
     else:
         logging.error("Not a supported OS type [%s]" % os)
         return None
@@ -337,12 +345,12 @@ def requests_get(url):
             resp = requests.get(url, verify=get_requests_verify())
             resp_status_code = resp.status_code
         except requests.exceptions.RequestException as e:
-            logging.warn("Retry count [%s] got exception: [%s]", rc, str(e))
+            logging.warning("Retry count [%s] got exception: [%s]", rc, str(e))
             if rc >= 10:
-                logging.warn("Max retries exceeded....giving up...")
+                logging.warning("Max retries exceeded....giving up...")
                 return None
             else:
-                logging.warn("Sleeping for [%s] seconds...", st)
+                logging.warning("Sleeping for [%s] seconds...", st)
                 time.sleep(st)
                 rc = rc + 1
                 st = st * 2
@@ -357,12 +365,12 @@ def requests_post(url, json):
             resp =  requests.post(url, json=json, verify=get_requests_verify())
             resp_status_code = resp.status_code
         except requests.exceptions.RequestException as e:
-            logging.warn("Retry count [%s] got exception: [%s]", rc, str(e))
+            logging.warning("Retry count [%s] got exception: [%s]", rc, str(e))
             if rc >= 10:
-                logging.warn("Max retries exceeded....giving up...")
+                logging.warning("Max retries exceeded....giving up...")
                 return None
             else:
-                logging.warn("Sleeping for [%s] seconds...", st)
+                logging.warning("Sleeping for [%s] seconds...", st)
                 time.sleep(st)
                 rc = rc + 1
                 st = st * 2
@@ -377,12 +385,12 @@ def requests_put(url, json):
             resp = requests.put(url, json=json, verify=get_requests_verify())
             resp_status_code = resp.status_code
         except requests.exceptions.RequestException as e:
-            logging.warn("Retry count [%s] got exception: [%s]", rc, str(e))
+            logging.warning("Retry count [%s] got exception: [%s]", rc, str(e))
             if rc >= 10:
-                logging.warn("Max retries exceeded....giving up...")
+                logging.warning("Max retries exceeded....giving up...")
                 return None
             else:
-                logging.warn("Sleeping for [%s] seconds...", st)
+                logging.warning("Sleeping for [%s] seconds...", st)
                 time.sleep(st)
                 rc = rc + 1
                 st = st * 2
@@ -400,8 +408,86 @@ def get_asset(asset_id, args):
         return None
 
 def tw_open(in_file, in_encoding, in_mode='rt'):
-    if sys.version_info[0] < 3:
-        f = open(in_file, mode=in_mode)
-    else:
-        f = open(in_file, mode=in_mode, encoding=in_encoding)
-    return f
+    try:
+        if sys.version_info[0] < 3:
+            f = open(in_file, mode=in_mode)
+        else:
+            f = open(in_file, mode=in_mode, encoding=in_encoding)
+        return f
+    except Exception as e:
+        logging.warning("Unable to open file ["+in_file+"]")
+        return None
+
+def set_run_args(args):
+    global run_args
+    run_args = args
+
+def get_run_args():
+    global run_args
+    return run_args
+
+def create_new_tool_run_record():
+    args = get_run_args()
+    if args.token is None or len(args.token) == 0:
+        return None
+    run_id  = args.mode if args.run_id is None else args.run_id
+    machine_id = get_ip()
+    tool_name = 'twigs'
+    tool_version = __version__
+    email = args.handle
+    url = "https://" + args.instance + "/api/v1/toolrun/new/"
+    auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
+    post_payload = { "run_id": run_id, "machine_id": machine_id, "email": email, "name": tool_name, "version": tool_version }
+    response = requests_post(url + auth_data, post_payload)
+    if response.status_code == 200:
+        args.run_record_id = response.json()['id']
+    return response
+
+def update_tool_run_record(status='RUNNING', observations=None):
+    args = get_run_args()
+    if args.token is None or len(args.token) == 0:
+        return None
+    run_record_id = args.run_record_id
+    url = "https://" + args.instance + "/api/v1/toolrun/" + str(run_record_id) + "/"
+    auth_data = "?handle=" + args.handle + "&token=" + args.token + "&format=json"
+    if status != 'RUNNING':
+        logging.shutdown()
+        observations = { } if observations is None else observations
+        observations['log_snippet'] = tail('twigs.log', args.encoding, 100)
+    put_payload = { "status": status, "observations": observations }
+    response = requests_put(url + auth_data, put_payload)
+    return response
+
+def tw_exit(exit_code, observations = None):
+    status = 'SUCCESS' if exit_code == 0 else 'FAIL'
+    update_tool_run_record(status, observations)
+    sys.exit(exit_code)
+
+def tail(fn, encoding, lines=20):
+    with open(fn, 'rb') as f:
+        total_lines_wanted = lines
+
+        BLOCK_SIZE = 1024
+        f.seek(0, 2)
+        block_end_byte = f.tell()
+        lines_to_go = total_lines_wanted
+        block_number = -1
+        blocks = [] # blocks of size BLOCK_SIZE, in reverse order starting
+                    # from the end of the file
+        while lines_to_go > 0 and block_end_byte > 0:
+            if (block_end_byte - BLOCK_SIZE > 0):
+                # read the last block we haven't yet read
+                f.seek(block_number*BLOCK_SIZE, 2)
+                blocks.append(f.read(BLOCK_SIZE).decode(encoding))
+            else:
+                # file too small, start from begining
+                f.seek(0,0)
+                # only read what was not read
+                blocks.append(f.read(block_end_byte).decode(encoding))
+            lines_found = blocks[-1].count('\n')
+            lines_to_go -= lines_found
+            block_end_byte -= BLOCK_SIZE
+            block_number -= 1
+        all_read_text = ''.join(reversed(blocks))
+        return '\n'.join(all_read_text.splitlines()[-total_lines_wanted:])
+
